/*
    This file is part of PBP.

    PBP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PBP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * pattern.cpp
 *
 *  Created on: Jan 4, 2013
 *      Author: Rustem Takhanov
 */

#include <stdio.h>
#include "pattern.h"
#include "math.h"
#include <cstdlib>

const int INFTY = 1E+6;

Potential::Potential(int number_of_variables, int alphabet_size, int max_num_words)
{
	n = number_of_variables;
	D = alphabet_size;
	max_nWords = max_num_words;
	pattern_costs = new double*[max_nWords];
	for(int i=0; i<max_nWords;i++)
	{
		pattern_costs[i] = new double[n+1];
		for(int j=0; j<=n;j++)
		{
			pattern_costs[i][j] = 0.0;
		}
	}
	nWords = 0;
	L = 0;
	max_length = 0;
//pointers
	stat_messages_forward = 0;
	stat_messages_backward = 0;
	stat_messages_forward_log = 0;
	stat_messages_backward_log = 0;
	map_messages = 0;
	marginals = 0;
	phi = 0;
	phi_back = 0;
	f = 0;

	for(int i=1;i<=D;i++)
	{
		wchar_t* p_str = new wchar_t[2];
		p_str[1] = L'\0';
		p_str[0] = (wchar_t) i;
		Cost c;
		c.i = 1;
		c.j = 1;
		c.alpha = p_str;
		c.cost = 0.0;
		add_pattern(&c);
	}
}

Potential::~Potential()
{
	delete [] pattern_costs;
}

void Potential::auf_null(int number_of_variables)
{
	n = number_of_variables;
	if(phi)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] phi[i];
			delete [] f[i];
		}
		delete [] phi;
		delete [] f;
	}
	if(phi_back)
	{
		for(int i=0;i<nSuffixes;i++)
		{
			delete [] phi_back[i];
		}
		delete [] phi_back;
	}
	if(stat_messages_forward)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] stat_messages_forward[i];
		}
		delete [] stat_messages_forward;
	}
	if(stat_messages_forward_log)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] stat_messages_forward_log[i];
		}
		delete [] stat_messages_forward_log;
	}
	if(stat_messages_backward)
	{
		for(int i=0;i<nSuffixes;i++)
		{
			delete [] stat_messages_backward[i];
		}
		delete [] stat_messages_backward;
	}
	if(stat_messages_backward_log)
	{
		for(int i=0;i<nSuffixes;i++)
		{
			delete [] stat_messages_backward_log[i];
		}
		delete [] stat_messages_backward_log;
	}
	if(marginals)
	{
		for(int i=0;i<nWords;i++)
		{
			delete [] marginals[i];
		}
		delete [] marginals;
	}
	if(map_messages)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] map_messages[i];
		}
		delete [] map_messages;
	}
	stat_messages_forward = 0;
	stat_messages_backward = 0;
	stat_messages_forward_log = 0;
	stat_messages_backward_log = 0;
	marginals = 0;
	map_messages = 0;
	phi = 0;
	phi_back = 0;
	f = 0;
	for(int i=0; i<max_nWords;i++)
	{
		delete [] pattern_costs[i];
	}
	delete [] pattern_costs;
	pattern_costs = new double*[max_nWords];
	for(int i=0; i<max_nWords;i++)
	{
		pattern_costs[i] = new double[n+1];
		for(int j=0; j<=n;j++)
		{
			pattern_costs[i][j] = 0.0;
		}
	}
}

int Potential::add_pattern(Cost* p)
{
	if(p != NULL)
	{
		int length = p->j - p->i + 1;
		if(length <= 0)
			return -1;
		wstring word(p->alpha,length);
		Map::iterator got = word_to_id_hash.find (word);
		if(got != word_to_id_hash.end())
		{
			pattern_costs[got->second][p->j] += p->cost;
		}
		else
		{
			word_to_id_hash[word] = nWords;
			pattern_costs[nWords][p->j] = p->cost;
			nWords++;
			max_length = (length > max_length)?length:max_length;
			L = L + length;
		}
	}
	return 0;
}

double Potential::value(wchar_t* x)
{
	double value = 0.0;
	for(int s=0;s<n;s++)
	{
		for(int i=max_length-1;i>=0;i--)
		{
			if(s-i<0)
			{
				continue;
			}
			wstring word(&(x[s-i]),i+1);
			Map::iterator got = word_to_id_hash.find(word);
			if(got != word_to_id_hash.end())
			{
				value = value + phi[got->second][s+1];
				break;
			}
		}
	}
	return value;
}


int Potential::words_preprocess()
{
	index_prefixes();
	compute_G_Pi();
	order_ids();
	compute_long_suf();

	index_suffixes();
	compute_G_Pi_back();
	order_ids_back();
	compute_long_pref();

	index_sufprefs();
	compute_children_sufpref();
	order_ids_sufpref();

	compute_T();

	return 0;
}

int Potential::weights_preprocess()
{
	compute_phi();
	compute_phi_back();
	compute_f();

	return 0;
}

void Potential::index_prefixes()
{
	Map::iterator it;
	id_to_prefix = new wstring[L+D+1];
	for(it=word_to_id_hash.begin(); it != word_to_id_hash.end(); it++)
	{
		prefix_to_id_hash[it->first] = it->second;
		id_to_prefix[it->second] = it->first;
	}
	nPrefixes = nWords;
	prefix_to_id_hash[L""] = nPrefixes;
	id_to_prefix[nPrefixes] = L"";
	nPrefixes++;
	minus_id = new int[L];
	minus_id[nWords] = nWords;
	for(it=word_to_id_hash.begin(); it != word_to_id_hash.end(); it++)
	{
		int length = (it->first).length();
		int prev_id = it->second;
		for(int i=length-1;i>=0;i--)
		{
			wstring prefix = (it->first).substr(0,i);
			Map::iterator got = prefix_to_id_hash.find (prefix);
			if(got == prefix_to_id_hash.end())
			{
				prefix_to_id_hash[prefix] = nPrefixes;
				id_to_prefix[nPrefixes] = prefix;
				minus_id[prev_id] = nPrefixes;
				prev_id = nPrefixes;
				nPrefixes++;
			}
			else
			{
				minus_id[prev_id] = got->second;
				prev_id = got->second;
			}
		}
	}
}

void Potential::index_suffixes()
{
	Map::iterator it;
	id_to_suffix = new wstring[L+D+1];
	for(it=word_to_id_hash.begin(); it != word_to_id_hash.end(); it++)
	{
		suffix_to_id_hash[it->first] = it->second;
		id_to_suffix[it->second] = it->first;
	}
	nSuffixes = nWords;
	suffix_to_id_hash[L""] = nSuffixes;
	id_to_suffix[nSuffixes] = L"";
	nSuffixes++;
	minus_id_back = new int[L];
	minus_id_back[nWords] = nWords;
	for(it=word_to_id_hash.begin(); it != word_to_id_hash.end(); it++)
	{
		int length = (it->first).length();
		int prev_id = it->second;
		for(int i=length-1;i>=0;i--)
		{
			wstring suffix = (it->first).substr(length-i,i);
			Map::iterator got = suffix_to_id_hash.find (suffix);
			if(got == suffix_to_id_hash.end())
			{
				suffix_to_id_hash[suffix] = nSuffixes;
				id_to_suffix[nSuffixes] = suffix;
				minus_id_back[prev_id] = nSuffixes;
				prev_id = nSuffixes;
				nSuffixes++;
			}
			else
			{
				minus_id_back[prev_id] = got->second;
				prev_id = got->second;
			}
		}
	}
}

void Potential::index_sufprefs()
{
	Map::iterator it;
	nSufprefs = 0;
	id_to_pref_id = new int[nPrefixes];
	id_to_suf_id = new int[nSuffixes];
	for(it=prefix_to_id_hash.begin(); it != prefix_to_id_hash.end(); it++)
	{
		Map::iterator got = suffix_to_id_hash.find(it->first);
		if(got != suffix_to_id_hash.end())
		{
			sufpref_to_id_hash[it->first] = nSufprefs;
			id_to_pref_id[nSufprefs] = it->second;
			id_to_suf_id[nSufprefs] = got->second;
			nSufprefs++;
		}
	}
}
// constructing of children
void Potential::compute_G_Pi()
{
	children = new int*[nPrefixes];
	for(int i=0; i<nPrefixes; i++)
	{
		children[i] = new int[nPrefixes];
		children[i][0] = 0;
	}
	Map::iterator it;
	for(it = prefix_to_id_hash.begin();it != prefix_to_id_hash.end();it++)
	{
		int length = (it->first).length();
		for(int i = length-1; i>=0;i--)
		{
			wstring suffix = (it->first).substr(length-i,i);
			Map::iterator got = prefix_to_id_hash.find(suffix);
			if(got != prefix_to_id_hash.end())
			{
				children[got->second][0]++;
				children[got->second][children[got->second][0]] = it->second;
				break;
			}
		}
	}
}

void Potential::compute_G_Pi_back()
{
	children_back = new int*[nSuffixes];
	for(int i=0; i<nSuffixes; i++)
	{
		children_back[i] = new int[nSuffixes];
		children_back[i][0] = 0;
	}
	Map::iterator it;
	for(it = suffix_to_id_hash.begin();it != suffix_to_id_hash.end();it++)
	{
		int length = (it->first).length();
		for(int i = length-1; i>=0;i--)
		{
			wstring prefix = (it->first).substr(0,i);
			Map::iterator got = suffix_to_id_hash.find(prefix);
			if(got != prefix_to_id_hash.end())
			{
				children_back[got->second][0]++;
				children_back[got->second][children_back[got->second][0]] = it->second;
				break;
			}
		}
	}
}

void Potential::compute_children_sufpref()
{
	Map::iterator it;
	children_sufpref = new int*[nSufprefs];
	offset_sufpref = new int*[nSufprefs];
	for(int id=0;id<nSufprefs;id++)
	{
		children_sufpref[id] = new int[L+1];
		offset_sufpref[id] = new int[L+1];
		children_sufpref[id][0] = 0;
	}
	for(it=sufpref_to_id_hash.begin(); it != sufpref_to_id_hash.end(); it++)
	{
		wstring word = it->first;
		int length = word.length();
		int max_right = 1;
		for(int i=1;i<=length;i++)
		{
			int max_right_l = 1;
			int first = 1;
			for(int j=length;j>=i;j--)
			{
				wstring subword = word.substr(i-1,j-i+1);
				Map::iterator got = sufpref_to_id_hash.find(subword);
				if((got != sufpref_to_id_hash.end()) && (j>=max_right))
				{
					if((i!=1) && (j!=length))
					{
						children_sufpref[got->second][0]++;
						children_sufpref[got->second][children_sufpref[got->second][0]] = it->second;
						offset_sufpref[got->second][children_sufpref[got->second][0]] = i-1;
					}
					if(first && ((i!=1) || (j!=length)))
					{
						first = 0;
						max_right_l = j;
					}
				}
			}
			if(max_right_l > max_right)
			{
				max_right = max_right_l;
			}
		}
	}
}

void Potential::order_ids()
{
	order = new int[nPrefixes];
	nomer = nPrefixes-1;
	order_subtree_ids(nWords);
}

void Potential::order_subtree_ids(int root_id)
{
	order[nomer] = root_id;
	nomer--;
	for(int i=1;i<=children[root_id][0];i++)
	{
		order_subtree_ids(children[root_id][i]);
	}
}

void Potential::order_ids_back()
{
	order_back = new int[nSuffixes];
	nomer = nSuffixes-1;
	order_subtree_ids_back(nWords);
}

void Potential::order_subtree_ids_back(int root_id)
{
	order_back[nomer] = root_id;
	nomer--;
	for(int i=1;i<=children_back[root_id][0];i++)
	{
		order_subtree_ids_back(children_back[root_id][i]);
	}
}

void Potential::order_ids_sufpref()
{
	order_sufpref = new int[nSufprefs-1];
	int cur_nomer = 0;
	for(int i=max_length;i>=1;i--)
	{
		for(Map::iterator it = sufpref_to_id_hash.begin(); it!=sufpref_to_id_hash.end(); it++)
		{
			if((it->first).length() == (unsigned int)i)
			{
				order_sufpref[cur_nomer] = it->second;
				cur_nomer++;
			}
		}
	}
}


void Potential::compute_T()
{
	int** descend = new int*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		descend[id] = new int[nPrefixes];
		int size = id_to_prefix[id].size();
		for(int id2=0;id2<nPrefixes;id2++)
		{
			int size2 = id_to_prefix[id2].size();
			if((size2>=size) && (id_to_prefix[id2].substr(size2-size,size) == id_to_prefix[id]))
			{
				descend[id][id2] = 1;
			}
			else
			{
				descend[id][id2] = 0;
			}
		}
	}
	T = new int*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		if(id == nWords)
		{
			T[id] = new int[1];
			T[id][0] = 0;
			continue;
		}
		T[id] = new int[nPrefixes+1];
		T[id][0] = 0;
		int alpha_minus = minus_id[id];
		int* c = new int[nPrefixes];
		for(int j=0;j<nPrefixes;j++)
		{
			if(descend[alpha_minus][j] == 1)
			{
				c[j] = 1;
			}
			else
			{
				c[j] = 0;
			}
		}
		for(int i=1;i<=children[id][0];i++)
		{
			int beta_minus = minus_id[children[id][i]];
			for(int j=0;j<nPrefixes;j++)
			{
				if(descend[beta_minus][j] == 1)
				{
					c[j] = 0;
				}
			}
		}
		for(int j=0;j<nPrefixes;j++)
		{
			if(c[j] == 1)
			{
				T[id][0]++;
				T[id][T[id][0]] = j;
			}
		}
		delete [] c;
	}
	for(int id=0;id<nPrefixes;id++)
	{
		delete [] descend[id];
	}
	delete [] descend;
}



// compute
void Potential::compute_phi()
{
	phi = new double*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		phi[id] = new double[n+1];
		for(int s=0;s<=n;s++)
		{
			phi[id][s] = 0.0;
		}
	}
	for(int s=1;s<=n;s++)
	{
		phi[nWords][s] = 0.0;
		compute_phi_id_s(nWords, s);
	}
}

void Potential::compute_phi_id_s(int id, int s)
{
	for(int c=1;c<=children[id][0];c++)
	{
		int child_id = children[id][c];
		double self_add = 0.0;
		if(child_id < nWords)
		{
			self_add = pattern_costs[child_id][s];
		}
		phi[child_id][s] = self_add + phi[id][s];
		compute_phi_id_s(child_id,s);
	}
}

void Potential::compute_f()
{
	f = new double*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		f[id] = new double[n+1];
		f[id][0] = INFTY;
	}
	f[nWords][0] = 0.0;
	for(int s=1;s<=n;s++)
	{
		for(int id=0;id<nPrefixes;id++)
		{
			f[id][s] = f[minus_id[id]][s-1]+phi[id][s];
		}
		f[nWords][s] = 0.0;
	}
	for(int s=0;s<=n;s++)
	{
		for(int id=0;id<nPrefixes;id++)
		{
			if(f[id][s] >= INFTY/2)
			{
				f[id][s] = 0.0;
			}
		}
	}
}


// compute
void Potential::compute_phi_back()
{
	phi_back = new double*[nSuffixes];
	for(int id=0;id<nSuffixes;id++)
	{
		phi_back[id] = new double[n+1];
		for(int s=0;s<=n;s++)
		{
			phi_back[id][s] = 0.0;
		}
	}
	for(int s=1;s<=n;s++)
	{
		phi_back[nWords][s] = 0.0;
		compute_phi_id_s_back(nWords, s);
	}
}

void Potential::compute_phi_id_s_back(int id, int s)
{
	for(int c=1;c<=children_back[id][0];c++)
	{
		int child_id = children_back[id][c];
		double self_add = 0.0;
		if(child_id < nWords)
		{
			int right_bound = s+id_to_suffix[child_id].length()-1;
			if(right_bound <= n)
			{
				self_add = pattern_costs[child_id][right_bound];
			}
		}
		phi_back[child_id][s] = self_add + phi_back[id][s];
		compute_phi_id_s_back(child_id,s);
	}
}

void Potential::compute_long_suf()
{
	long_suf = new int*[nPrefixes];
	in_Pi =  new int*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		long_suf[id] = new int[D+1];
		in_Pi[id] = new int[D+1];
	}
	Map::iterator it;
	for(it = prefix_to_id_hash.begin();it != prefix_to_id_hash.end();it++)
	{
		for(int d=1;d<=D;d++)
		{
			wchar_t d_str[2];
			d_str[0] = (wchar_t)d;
			d_str[1] = L'\0';
			wstring alpha_d = it->first+d_str;
			Map::iterator got = prefix_to_id_hash.find(alpha_d);
			if(got != prefix_to_id_hash.end())
			{
				in_Pi[it->second][d] = 1;
			}
			else
			{
				in_Pi[it->second][d] = 0;
			}
			if(!in_Pi[it->second][d])
			{
				int length = alpha_d.length();
				for(int i = length-1; i>=0;i--)
				{
					wstring suffix = alpha_d.substr(length-i,i);
					got = prefix_to_id_hash.find(suffix);
					if(got != prefix_to_id_hash.end())
						break;
				}
			}
			long_suf[it->second][d] = got->second;
		}
	}
}

void Potential::compute_long_pref()
{
	long_pref = new int*[nSuffixes];
	in_Pi_back =  new int*[nSuffixes];
	for(int id=0;id<nSuffixes;id++)
	{
		long_pref[id] = new int[D+1];
		in_Pi_back[id] = new int[D+1];
	}
	Map::iterator it;
	for(it = suffix_to_id_hash.begin();it != suffix_to_id_hash.end();it++)
	{
		for(int d=1;d<=D;d++)
		{
			wchar_t d_str[2];
			d_str[0] = (wchar_t)d;
			d_str[1] = L'\0';
			wstring d_alpha = d_str+it->first;
			Map::iterator got = suffix_to_id_hash.find(d_alpha);
			if(got != suffix_to_id_hash.end())
			{
				in_Pi_back[it->second][d] = 1;
			}
			else
			{
				in_Pi_back[it->second][d] = 0;
			}
			if(!in_Pi_back[it->second][d])
			{
				int length = d_alpha.length();
				for(int i = length-1; i>=0;i--)
				{
					wstring prefix = d_alpha.substr(0,i);
					got = suffix_to_id_hash.find(prefix);
					if(got != prefix_to_id_hash.end())
						break;
				}
			}
			long_pref[it->second][d] = got->second;
		}
	}
}

double Potential::compute_stat_sum()
{
	stat_messages_forward = new double*[nPrefixes];
	stat_messages_forward_log = new double*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		stat_messages_forward[id] = new double[n+1];
		stat_messages_forward[id][0] = 1.0;
		stat_messages_forward_log[id] = new double[n+1];
		stat_messages_forward_log[id][0] = -INFTY;
	}
	stat_messages_forward_log[nWords][0] = 0.0;

	double auxiliary_stat_message=0;
	double auxiliary_stat_message_log;
	double max_child_value;

	for(int s=1;s<=n;s++)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			int id = order[i];
			if((id != nWords) && (stat_messages_forward_log[minus_id[id]][s-1] == -INFTY))
			{
				stat_messages_forward_log[id][s] = -INFTY;
				stat_messages_forward[id][s] = 1.0;
				continue;
			}
			if(id != nWords)
			{
				auxiliary_stat_message_log = stat_messages_forward_log[minus_id[id]][s-1] - phi[id][s];
				auxiliary_stat_message = stat_messages_forward[minus_id[id]][s-1];
				for(int c=1;c<=children[id][0];c++)
				{
					auxiliary_stat_message -= stat_messages_forward[minus_id[children[id][c]]][s-1]*exp(stat_messages_forward_log[minus_id[children[id][c]]][s-1]-stat_messages_forward_log[minus_id[id]][s-1]);
				}
				if(auxiliary_stat_message<0)
				{
					auxiliary_stat_message_log = -INFTY;
					auxiliary_stat_message = 1.0;
				}
			}
			else
			{
				auxiliary_stat_message_log = -INFTY;
				auxiliary_stat_message = 1.0;
			}
			max_child_value = -INFTY;
			for(int c=1;c<=children[id][0];c++)
			{
				max_child_value = (stat_messages_forward_log[children[id][c]][s] > max_child_value)?stat_messages_forward_log[children[id][c]][s]:max_child_value;
			}
			max_child_value = (auxiliary_stat_message_log > max_child_value)?auxiliary_stat_message_log:max_child_value;
			if(max_child_value < -INFTY/2)
			{
				stat_messages_forward_log[id][s] = -INFTY;
				stat_messages_forward[id][s] = 1.0;
				continue;
			}
			stat_messages_forward[id][s] = auxiliary_stat_message*exp(auxiliary_stat_message_log-max_child_value);
			for(int c=1;c<=children[id][0];c++)
			{
				stat_messages_forward[id][s] += stat_messages_forward[children[id][c]][s]*exp(stat_messages_forward_log[children[id][c]][s]-max_child_value);
			}
			if(stat_messages_forward[id][s] > 1.0E+100)
			{
				stat_messages_forward_log[id][s] = max_child_value+log(children[id][0]+1);
				stat_messages_forward[id][s] /= (children[id][0]+1);
			}
			else
			{
				stat_messages_forward_log[id][s] = max_child_value;
			}
		}
	}
	return log(stat_messages_forward[nWords][n])+stat_messages_forward_log[nWords][n];
}


double Potential::compute_stat_sum_back()
{
	stat_messages_backward = new double*[nSuffixes];
	stat_messages_backward_log = new double*[nSuffixes];

	for(int id=0;id<nSuffixes;id++)
	{
		stat_messages_backward[id] = new double[n+2];
		stat_messages_backward[id][n+1] = 1.0;
		stat_messages_backward_log[id] = new double[n+2];
		stat_messages_backward_log[id][n+1] = -INFTY;
	}
	stat_messages_backward_log[nWords][n+1] = 0.0;

	double auxiliary_stat_message=0;
	double auxiliary_stat_message_log;
	double max_child_value;

	for(int s=n;s>=1;s--)
	{
		for(int i=0;i<nSuffixes;i++)
		{
			int id = order_back[i];
			if((id != nWords) && (stat_messages_backward_log[minus_id_back[id]][s+1] == -INFTY))
			{
				stat_messages_backward_log[id][s] = -INFTY;
				stat_messages_backward[id][s] = 1.0;
				continue;
			}
			if(id != nWords)
			{
				auxiliary_stat_message = stat_messages_backward[minus_id_back[id]][s+1];
				auxiliary_stat_message_log = stat_messages_backward_log[minus_id_back[id]][s+1] - phi_back[id][s];
				for(int c=1;c<=children_back[id][0];c++)
				{
					auxiliary_stat_message -= stat_messages_backward[minus_id_back[children_back[id][c]]][s+1]*exp(stat_messages_backward_log[minus_id_back[children_back[id][c]]][s+1]-stat_messages_backward_log[minus_id_back[id]][s+1]);
				}
				if(auxiliary_stat_message<0)
				{
					auxiliary_stat_message_log = -INFTY;
					auxiliary_stat_message = 1.0;
				}
			}
			else
			{
				auxiliary_stat_message_log = -INFTY;
				auxiliary_stat_message = 1.0;
			}
			max_child_value = -INFTY;
			for(int c=1;c<=children_back[id][0];c++)
			{
				max_child_value = (stat_messages_backward_log[children_back[id][c]][s] > max_child_value)?stat_messages_backward_log[children_back[id][c]][s]:max_child_value;
			}
			max_child_value = (auxiliary_stat_message_log > max_child_value)?auxiliary_stat_message_log:max_child_value;
			if(max_child_value < -INFTY/2)
			{
				stat_messages_backward_log[id][s] = -INFTY;
				stat_messages_backward[id][s] = 1.0;
				continue;
			}
			stat_messages_backward[id][s] = auxiliary_stat_message*exp(auxiliary_stat_message_log-max_child_value);
			for(int c=1;c<=children_back[id][0];c++)
			{
				stat_messages_backward[id][s] += stat_messages_backward[children_back[id][c]][s]*exp(stat_messages_backward_log[children_back[id][c]][s]-max_child_value);
			}
			if(stat_messages_backward[id][s] > 1.0E+100)
			{
				stat_messages_backward_log[id][s] = max_child_value+log(children_back[id][0]+1);
				stat_messages_backward[id][s] /= (children_back[id][0]+1);
			}
			else
			{
				stat_messages_backward_log[id][s] = max_child_value;
			}
		}
	}
	return log(stat_messages_backward[nWords][1])+stat_messages_backward_log[nWords][1];
}

int Potential::compute_marginals()
{
	double logZ2 = compute_stat_sum_back();
	logZ = compute_stat_sum();
/*	printf("logZ1-logZ2=%f\n", logZ2 - logZ);
	if(logZ2/logZ - 1 > 0.00001 || logZ2/logZ - 1 < -0.00001 || !logZ)
	{
		printf("something wrong with ratio %f %f %f %0.200f\n", logZ2/logZ - 1, logZ2, logZ, stat_messages_backward[nWords][1]);
	}*/
	marginals = new double*[nWords];
	for(int id=0;id<nWords;id++)
	{
		marginals[id] = new double[n+1];
	}
	double** raw_marginals = new double*[nSufprefs];
	for(int id=0;id<nSufprefs;id++)
	{
		raw_marginals[id] = new double[n+1];
	}

	for(int i=0;i<nSufprefs-1;i++)
	{
		int id = order_sufpref[i];
		for(int s=1;s<=n;s++)
		{
			int left_s = s-id_to_prefix[id_to_pref_id[id]].length()+1;
			if(left_s<1)
			{
				raw_marginals[id][s] = -1;
				continue;
			}
			raw_marginals[id][s] = stat_messages_forward[id_to_pref_id[id]][s]*stat_messages_backward[id_to_suf_id[id]][left_s]*exp(stat_messages_forward_log[id_to_pref_id[id]][s]+stat_messages_backward_log[id_to_suf_id[id]][left_s]-logZ+f[id_to_pref_id[id]][s]);
			for(int c=1;c<=children_sufpref[id][0];c++)
			{
				int left = left_s - offset_sufpref[id][c];
				int pref_id = id_to_pref_id[children_sufpref[id][c]];
				int suf_id = id_to_suf_id[children_sufpref[id][c]];
				int right = left+id_to_prefix[pref_id].length()-1;
				if(left<1 || right > n)
				{
					continue;
				}
				raw_marginals[id][s] += raw_marginals[children_sufpref[id][c]][right];
				double subtract;
				if(pref_id < nWords)
				{
					subtract = stat_messages_forward[minus_id[pref_id]][right-1]*stat_messages_backward[minus_id_back[suf_id]][left+1]*exp(stat_messages_forward_log[minus_id[pref_id]][right-1]+stat_messages_backward_log[minus_id_back[suf_id]][left+1]-
							phi[pref_id][right]-phi_back[suf_id][left]+f[pref_id][right]-logZ+pattern_costs[pref_id][right]);
				}
				else
				{
					subtract = stat_messages_forward[minus_id[pref_id]][right-1]*stat_messages_backward[minus_id_back[suf_id]][left+1]*exp(stat_messages_forward_log[minus_id[pref_id]][right-1]+stat_messages_backward_log[minus_id_back[suf_id]][left+1]-
							phi[pref_id][right]-phi_back[suf_id][left]+f[pref_id][right]-logZ);
				}
				raw_marginals[id][s] -= subtract;
			}
		}
	}
//	Cost* marginals_list = new Cost[nWords*n];
	for(int id=0;id<nSufprefs;id++)
	{
		int pref_id = id_to_pref_id[id];
		if(pref_id >= nWords)
			continue;
		for(int s=1;s<=n;s++)
		{
			marginals[pref_id][s] = raw_marginals[id][s];
/*			int left_s = s - id_to_prefix[pref_id].length() + 1;
			if(left_s>=1)
			{
				marginals_list[i].i = left_s;
				marginals_list[i].j = s;
				marginals_list[i].cost = marginals[pref_id][s];
				wchar_t* word = new wchar_t[id_to_prefix[pref_id].length()+1];
				wcscpy(word, id_to_prefix[pref_id].c_str());
				marginals_list[i].alpha = word;
				i++;
			}*/
		}
	}
//	marginals_list[i].i = 0;
//	marginals_list[i].cost = logZ;

	for(int id=0;id<nSufprefs;id++)
	{
		delete [] raw_marginals[id];
	}
	delete [] raw_marginals;
	return 0;
}

/*void Potential::compute_extended_G_Pi()
{
	int piD = D*nPrefixes+1;
	extended_children = new int*[piD];
	for(int id=0;id<nPrefixes;id++)
	{
		for(int i=0;i<D;i++)
		{
			int new_id = id*D+i;
			extended_children[new_id] = new int[children[id][0]+1];
			extended_children[new_id][0] = children[id][0];
			for(int j=1;j<=extended_children[new_id][0];j++)
			{
				extended_children[new_id][j] = children[id][j]*D+i;
			}
		}
	}
	isWord = new int[piD];
	for(int i=0;i<piD;i++)
	{
		isWord[i] = 0;
	}
	Map::iterator it;
	for(it=word_to_id_hash.begin(); it != word_to_id_hash.end(); it++)
	{
		wstring s = it->first;
		int length = s.length();
		int prefix_id = prefix_to_id_hash[s.substr(0,length-1)];
		int last_letter = (int)*(s[length-1]);
		isWord[prefix_id*D+last_letter]=1;
	}

}*/

wchar_t* Potential::compute_map()
{
	int s;
	map_messages =new double*[nPrefixes];
	int ***to_the_left = new int**[nPrefixes];
	int *empty_word_to_the_left = new int[n+1];
	for(int id=0;id<nPrefixes;id++)
	{
		map_messages[id] = new double[n+1];
		map_messages[id][0] = -INFTY;
		to_the_left[id] = new int*[D+1];
		for(int d=1;d<=D;d++)
		{
			to_the_left[id][d] = new int[n+1];
		}
	}
	map_messages[nWords][0] = 0.0;

	for(s=1;s<=n;s++)
	{
		double **temp_map_messages = new double*[nPrefixes];
		for(int i=0;i<nPrefixes;i++)
		{
			int id = order[i];
			temp_map_messages[id] = new double[D+1];
			for(int d=1;d<=D;d++)
			{
				temp_map_messages[id][d] = phi[long_suf[id][d]][s]+map_messages[id][s-1];
				to_the_left[id][d][s] = id;
				for(int c=1;c<=children[id][0];c++)
				{
					if((!in_Pi[children[id][c]][d]) && (temp_map_messages[children[id][c]][d]<temp_map_messages[id][d]))
					{
						temp_map_messages[id][d] = temp_map_messages[children[id][c]][d];
						to_the_left[id][d][s] = children[id][c];
					}
				}
			}
		}
		map_messages[nWords][s] = INFTY;
		empty_word_to_the_left[s] = -1;
		for(int d=1;d<=D;d++)
		{
			if(!in_Pi[nWords][d])
			{
				if(temp_map_messages[nWords][d] < map_messages[nWords][s])
				{
					map_messages[nWords][s] = temp_map_messages[nWords][d];
					empty_word_to_the_left[s] = d;
				}
			}
		}

		Map::iterator it;
		for(it = prefix_to_id_hash.begin();it->second != nWords;it++)
		{
			int last = (int)*((it->first).rbegin());
			map_messages[it->second][s] = temp_map_messages[minus_id[it->second]][last];
		}
		for(it++;it != prefix_to_id_hash.end();it++)
		{
			int last = (int)*((it->first).rbegin());
			map_messages[it->second][s] = temp_map_messages[minus_id[it->second]][last];
		}
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] temp_map_messages[i];
		}
		delete [] temp_map_messages;
	}
	int id_max = 0;
	for(int i=0;i<nPrefixes;i++)
	{
		id_max = (map_messages[i][n]<map_messages[id_max][n])?i:id_max;
	}
	int cur_id;
	int cur_d;
	if(id_max!=nWords)
	{
		cur_id = minus_id[id_max];
		cur_d = (int)*(id_to_prefix[id_max].rbegin());
	}
	else
	{
		cur_id = nWords;
		cur_d = empty_word_to_the_left[n];
	}

	wchar_t* output = new wchar_t[n];
	s=n;
	while(s>0)
	{
		if(to_the_left[cur_id][cur_d][s] == cur_id)
		{
			output[s-1] = cur_d;
			if(cur_id!=nWords)
			{
				cur_d = (int)*(id_to_prefix[cur_id].rbegin());
				cur_id = minus_id[cur_id];
			}
			else
			{
				cur_id = nWords;
				cur_d = empty_word_to_the_left[s-1];
			}
			s--;
		}
		else
		{
			cur_id = to_the_left[cur_id][cur_d][s];
		}
	}
	delete [] empty_word_to_the_left;
	for(int id=0;id<nPrefixes;id++)
	{
		for(int d=1;d<=D;d++)
		{
			delete [] to_the_left[id][d];
		}
		delete [] to_the_left[id];
	}
	delete [] to_the_left;
	return output;
}

wchar_t* Potential::sample()
{
	compute_stat_sum_for_sample();
	int current;
	double value = 0.0;
	double value_log = -INFTY;
	for(int id=0;id<nPrefixes;id++)
	{
		value_log = (value_log<auxiliary_stat_message_log[id][n])?auxiliary_stat_message_log[id][n]:value_log;
	}
	for(int id=0;id<nPrefixes;id++)
	{
		value += auxiliary_stat_message[id][n]*exp(auxiliary_stat_message_log[id][n] - value_log);
	}
	double r = rand()*value/RAND_MAX;
	value = 0.0;
	for(int id=0;id<nPrefixes;id++)
	{
		value += auxiliary_stat_message[id][n]*exp(auxiliary_stat_message_log[id][n] - value_log);
		if(value>r)
		{
			current = id;
			break;
		}
	}
	wchar_t* out = new wchar_t[n];
	out[n-1] = *(id_to_prefix[current].rbegin());
	for(int s=n-1;s>=1;s--)
	{
		current = choose(s,current);
		out[s-1] = *(id_to_prefix[current].rbegin());
	}
	return out;
}

int Potential::choose(int s, int cur_id)
{
	double value = 0.0;
	double value_log = -INFTY;
	for(int i=1;i<=T[cur_id][0];i++)
	{
		value_log = (value_log<auxiliary_stat_message_log[T[cur_id][i]][s])?auxiliary_stat_message_log[T[cur_id][i]][s]:value_log;
	}
	for(int i=1;i<=T[cur_id][0];i++)
	{
		value += auxiliary_stat_message[T[cur_id][i]][s]*exp(auxiliary_stat_message_log[T[cur_id][i]][s] - value_log);
	}
	double r = rand()*value/RAND_MAX;
	value = 0.0;
	for(int i=1;i<=T[cur_id][0];i++)
	{
		value += auxiliary_stat_message[T[cur_id][i]][s]*exp(auxiliary_stat_message_log[T[cur_id][i]][s] - value_log);
		if(value>r)
		{
			return T[cur_id][i];
			break;
		}
	}
}

double Potential::compute_stat_sum_for_sample()
{
	if(stat_messages_forward)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] stat_messages_forward[i];
		}
		delete [] stat_messages_forward;
	}
	if(stat_messages_forward_log)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			delete [] stat_messages_forward_log[i];
		}
		delete [] stat_messages_forward_log;
	}
	stat_messages_forward = new double*[nPrefixes];
	stat_messages_forward_log = new double*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		stat_messages_forward[id] = new double[n+1];
		stat_messages_forward[id][0] = 1.0;
		stat_messages_forward_log[id] = new double[n+1];
		stat_messages_forward_log[id][0] = -INFTY;
	}
	stat_messages_forward_log[nWords][0] = 0.0;

	auxiliary_stat_message = new double*[nPrefixes];
	auxiliary_stat_message_log = new double*[nPrefixes];
	for(int id=0;id<nPrefixes;id++)
	{
		auxiliary_stat_message[id] = new double[n+1];
		auxiliary_stat_message_log[id] = new double[n+1];
	}

	double max_child_value;

	for(int s=1;s<=n;s++)
	{
		for(int i=0;i<nPrefixes;i++)
		{
			int id = order[i];
			if((id != nWords) && (stat_messages_forward_log[minus_id[id]][s-1] == -INFTY))
			{
				stat_messages_forward_log[id][s] = -INFTY;
				stat_messages_forward[id][s] = 1.0;
				auxiliary_stat_message_log[id][s] = -INFTY;
				auxiliary_stat_message[id][s] = 1.0;
				continue;
			}
			if(id != nWords)
			{
				auxiliary_stat_message_log[id][s] = stat_messages_forward_log[minus_id[id]][s-1] - phi[id][s];
				auxiliary_stat_message[id][s] = stat_messages_forward[minus_id[id]][s-1];
				for(int c=1;c<=children[id][0];c++)
				{
					auxiliary_stat_message[id][s] -= stat_messages_forward[minus_id[children[id][c]]][s-1]*exp(stat_messages_forward_log[minus_id[children[id][c]]][s-1]-stat_messages_forward_log[minus_id[id]][s-1]);
				}
				if(auxiliary_stat_message[id][s]<0)
				{
					auxiliary_stat_message_log[id][s] = -INFTY;
					auxiliary_stat_message[id][s] = 1.0;
				}
			}
			else
			{
				auxiliary_stat_message_log[id][s] = -INFTY;
				auxiliary_stat_message[id][s] = 1.0;
			}
			max_child_value = -INFTY;
			for(int c=1;c<=children[id][0];c++)
			{
				max_child_value = (stat_messages_forward_log[children[id][c]][s] > max_child_value)?stat_messages_forward_log[children[id][c]][s]:max_child_value;
			}
			max_child_value = (auxiliary_stat_message_log[id][s] > max_child_value)?auxiliary_stat_message_log[id][s]:max_child_value;
			if(max_child_value < -INFTY/2)
			{
				stat_messages_forward_log[id][s] = -INFTY;
				stat_messages_forward[id][s] = 1.0;
				continue;
			}
			stat_messages_forward[id][s] = auxiliary_stat_message[id][s]*exp(auxiliary_stat_message_log[id][s]-max_child_value);
			for(int c=1;c<=children[id][0];c++)
			{
				stat_messages_forward[id][s] += stat_messages_forward[children[id][c]][s]*exp(stat_messages_forward_log[children[id][c]][s]-max_child_value);
			}
			if(stat_messages_forward[id][s] > 1.0E+100)
			{
				stat_messages_forward_log[id][s] = max_child_value+log(children[id][0]+1);
				stat_messages_forward[id][s] /= (children[id][0]+1);
			}
			else
			{
				stat_messages_forward_log[id][s] = max_child_value;
			}
		}
	}
	return log(stat_messages_forward[nWords][n])+stat_messages_forward_log[nWords][n];
}
