/*
    This file is part of PBP.

    PBP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PBP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * pattern.h
 *
 *  Created on: Jan 4, 2013
 *      Author: Rustem Takhanov
 */

#ifndef PATTERN_H_
#define PATTERN_H_

#include <iostream>
#include <tr1/unordered_map>
#include <string>

using namespace std;
using namespace std::tr1;
typedef unordered_map<std::wstring,int> Map;

struct Cost
{
	int i,j;
	wchar_t* alpha;
	double cost; /* this parameter could be
					1)cost of a pattern,
					2)probability P(x_{i:j}=\alpha)
					3)or, message M_j(\alpha)*/
};


class Potential
{
public:
	Potential(int number_of_variables, int alphabet_size, int max_num_words);
	~Potential();
	void auf_null(int number_of_variables);

	int n;                                // number of variables
	int D;                                // alphabet size
	int max_nWords;                       // upper bound for nWords
	double** pattern_costs;               /* array of patterns` costs
	                                      pattern_costs[id,k] will be a cost of pattern that 1) ends at k
	                                      2) a string is equal to id_to_word[id] */
	Map word_to_id_hash;                  // hash that maps a word to its index

	double** stat_messages_forward;        // where we will save all messages for stat_sum (W_s(\alpha))
	double** stat_messages_forward_log;        // where we will save all messages for stat_sum (W_s(\alpha))
	double** stat_messages_backward;       // the same, but in backward direction
	double** stat_messages_backward_log;       // the same, but in backward direction
	double** map_messages;                 // W_s(\alpha) messages for MAP
	double** marginals;                    // marginals
	double logZ;

	int add_pattern(Cost* p);             //adding patterns with costs to functional
	int words_preprocess();               //makes all preprocessing with \Pi
	int weights_preprocess();             //makes all preprocessing with weights of patterns
	double value(wchar_t* x);           //return value of a potential on x
	double compute_stat_sum();            // Returning value is statsum
	double compute_stat_sum_back();       // Returning value is statsum computed by backward algorithm
	wchar_t* compute_map();             // returns optimal string
	int compute_marginals();              //return all marginals
	wchar_t* sample();
protected:
	// all these variables needed to speed up processing, so that instead of using words we will use their indices
//	wchar_t** id_to_word;               // inverse mapping
	int nWords;                           // # words involved in patterns
	int max_length;                       // maximum length of a word
	int L;                                // sum of word lengths

// ========================================================================================
//all data needed for forward algorithm

	Map prefix_to_id_hash;                // hash that maps a prefix to its index
	wstring* id_to_prefix;                // inverse mapping
	int nPrefixes;                        // # prefixes of words involved in patterns

	int* minus_id;                         // array that maps alpha id to alpha^- id
	int** children;                        /* array that maps alpha id to list of beta ids
	                                       such that (\alpha,\beta)\in E[\Pi_s] where \Pi_s are all prefixes of \Pi_0
	                                       Note: children[id][0] is equal to the number of such children */
	int* order;                            /* order[id] is a number assigned to id, such that
	                                       if alpha = *beta then order[alpha id] > order[beta id]*/
	double** phi;                          // phi_s(\alpha) will be saved as two-dimensional array phi[\alpha id][s]
	double** f;
	int** long_suf;                        // long_suf[\alpha id][d] is id of longest suffix of "alpha d" which belongs to \Pi_0
	int** in_Pi;
	int nomer;

	int* isWord;                           /* isWord[id] is 1, if id correspond to word in \Pi, and 0, if else */

//==========================================================================================
//data needed for backward algorithm

	Map suffix_to_id_hash;                // hash that maps a suffix to its index
	wstring* id_to_suffix;                // inverse mapping
	int nSuffixes;                        // # suffixes of words involved in patterns

	int* minus_id_back;                         // array that maps alpha id to alpha^- id
	int** children_back;                        /* array that maps alpha id to list of beta ids
	                                       such that (\alpha,\beta)\in E[\Pi_s] where \Pi_s are all prefixes of \Pi_0
	                                       Note: children[id][0] is equal to the number of such children */
	int* order_back;                            /* order[id] is a number assigned to id, such that
	                                       if alpha = *beta then order[alpha id] > order[beta id]*/
	double** phi_back;                          // phi_s(\alpha) will be saved as two-dimensional array phi[\alpha id][s]
	int** long_pref;                        // long_suf[\alpha id][d] is id of longest prefix of "d alpha" which belongs to \Pi_0
	int** in_Pi_back;

	int* isWord_back;                           /* isWord[id] is 1, if id correspond to word in \Pi, and 0, if else */
//=======================================================================================

	Map sufpref_to_id_hash;                // hash that maps a suffix to its index
	int* id_to_pref_id;
	int* id_to_suf_id;
	int nSufprefs;                        // # suffixes of words involved in patterns

	int** children_sufpref;                        /* array that maps alpha id to list of beta ids
	                                       such that (\alpha,\beta)\in E[\Pi_s] where \Pi_s are all prefixes of \Pi_0
	                                       Note: children[id][0] is equal to the number of such children */
	int** offset_sufpref;
	int* order_sufpref;                            /* order[id] is a number assigned to id, such that
	                                       if alpha = *beta then order[alpha id] > order[beta id]*/
//=======================================================================================
//needed for sampling
	int** T;
	double** auxiliary_stat_message;
	double** auxiliary_stat_message_log;
	void compute_T();
	int choose(int s, int cur_id);
	double compute_stat_sum_for_sample();
//=======================================================================================


	void index_prefixes();
	void compute_G_Pi();
	void order_ids();
	void order_subtree_ids(int root_id);
	void compute_phi();
	void compute_long_suf();
	void compute_phi_id_s(int id, int s);
//========================================================================================

	void index_suffixes();
	void compute_G_Pi_back();
	void order_ids_back();
	void order_subtree_ids_back(int root_id);
	void compute_phi_back();
	void compute_long_pref();
	void compute_phi_id_s_back(int id, int s);
//========================================================================================

	void index_sufprefs();
	void compute_children_sufpref();
	void compute_f();
	void order_ids_sufpref();

	//	void compute_extended_G_Pi();
	void compute_extended_phi();
	void compute_inclusion_for_marginals();
};

#endif /* PATTERN_H_ */
